# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from .cancnfcesubst import EnvEventoCancSubst, RetEnvEventoCancSubst, ProcEventoCancSubst
from pynfe.nfe.manual_600.cancnfe_evento import EnvEvento, RetEnvEvento, ProcEventoNFe
from pynfe.nfe.manual_600.inutnfe import InutNFe, RetInutNFe, ProcInutNFe
from pynfe.nfe.manual_600.consrecinfe import ConsReciNFe, RetConsReciNFe
from pynfe.nfe.manual_600.processador import ProcessoNFe, ProcessadorNFe
from pynfe.nfe.manual_600.envinfe import EnviNFe, RetEnviNFe
from pynfe.nfe.manual_600.conssitnfe import ConsSitNFe
from pynfe.nfce.manual_600.webservices_flag import *
from pynfe.xml_sped.certificado import Certificado
from .consrecinfce import RetConsSitNFe
from pynfe.xml_sped import UF_CODIGO
from datetime import datetime
from . import webservices
import tempfile
import logging
import pytz
import time
import os
import requests


class ProcessadorNFCe(ProcessadorNFe):
    """
    Processador da nota fiscal eletronica
    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    def __init__(self, certify=None, key=None, environment=2, state='SVRS', contingency='',
                 tentativas=20, timeout=10):
        super(ProcessadorNFCe, self).__init__(certify, key, environment, state, contingency)
        self.estado = state  # Inicialmente os estados utilizam o ambiente do Rio Grande do Sul
        self.certificado = Certificado()
        self.certificado.certificado = certify
        self.certificado.chave = key
        self.certificado.cert_str = certify
        self.certificado.key_str = key
        self.ws = webservices.ESTADO_WS[self.versao][self.estado or 'SVRS']
        self.tentativas = tentativas or 20
        self.timeout = timeout or 10
        self.logger = logging.getLogger('SEFAZ-NFCe')  # Registro de log

    @staticmethod
    def remove_accents(text, norm='NFKD', cat=None):
        from unicodedata import normalize, category
        cat = cat or ['Mn', 'Cc']
        text = ''.join(
            c for c in normalize(norm, text)
            if category(c) not in cat)
        return text.encode('ascii', 'replace').decode('utf-8')

    def __init_soap_envio(self, servico, envio):
        self._soap_envio.webservice = self.ws[self.ambiente][servico]['webservice']
        self._soap_envio.metodo = self.ws[self.ambiente][servico]['metodo']
        self._soap_envio.cUF = UF_CODIGO[self.estado]
        self._soap_envio.namespace = self.ws[self.ambiente][servico]['namespace']
        self._soap_envio.envio = envio

    def __init_soap_retorno(self, servico, resposta):
        self._soap_retorno.webservice = self.ws[self.ambiente][servico]['webservice']
        self._soap_retorno.metodo = self.ws[self.ambiente][servico]['metodo']
        self._soap_retorno.resposta = resposta

    def __check_cert_validate(self):
        today = datetime.now(pytz.timezone('UTC')).astimezone(pytz.timezone('Brazil/East'))
        validate = self.certificado.data_fim_validade
        if validate.strftime('%Y%m%d') < today.strftime('%Y%m%d'):
            return False

        return True

    def __init_certify(self, cert, key):
        self.certificado.cert_str = self.certificado.certificado
        self.certificado.key_str = self.certificado.chave
        self.certificado.prepara_certificado_arquivo_pfx()

    def _conectar_servico(self, servico, envio, resposta):
        self.logger.info('>> %s-%s v-%s' % (
            self.ambiente, 'Producao' if self.ambiente == 1 else 'Homologacao', self.versao))
        self.logger.info(self.contingencia or '>> %s/%s' % (self.estado, UF_CODIGO[self.estado]))
        list(map(lambda m: self.logger.info('>> %s: %s' % m), list(self.ws[self.ambiente][servico].items())))
        self.__init_soap_envio(servico, envio)
        self.__init_soap_retorno(servico, resposta)
        _servidor = self.ws[self.ambiente][servico]['servidor']
        _url = self.ws[self.ambiente][servico]['url']

        # Verifica vencimento do certificado
        cert, key = self.certificado.certificado, self.certificado.chave
        self.__init_certify(cert, key)
        if not self.__check_cert_validate():
            self._soap_retorno.resposta.cStat.valor = '999'
            self._soap_retorno.resposta.xMotivo.valor = 'Certificado digital com data expirada.'
            self.logger.critical(self._soap_retorno.resposta.xMotivo.valor)

        # Arquivos de chave e certificado digital que serao utilizados na conexao https
        nome_arq_chave = tempfile.NamedTemporaryFile('w', dir='/dev/shm')
        nome_arq_chave.write(self.certificado.chave)
        nome_arq_chave.flush()
        nome_arq_certificado = tempfile.NamedTemporaryFile('w', dir='/dev/shm')
        nome_arq_certificado.write(self.certificado.certificado)
        nome_arq_certificado.flush()
        nome_arq_cadeia_cert = open(os.path.dirname(os.path.abspath(__file__)) + '/../../xml_sped/cadeia.pem')

        webservice = webservices.ESTADO_WS[self.versao][self.contingencia or self.estado]
        webservice = webservice[self.ambiente][servico]
        self._soap_envio.webservice = webservice['webservice']
        self._soap_envio.metodo = webservice['metodo']
        self._soap_envio.cUF = UF_CODIGO[self.estado]
        self._soap_envio.namespace = webservice['namespace']
        self._soap_envio.envio = envio
        self._servidor = webservice['servidor']
        self._url = webservice['url']
        self.logger.info('>> Metodo da requisicao: POST')
        self.logger.info('>> XML envio: %s' % self._soap_envio.xml)

        for tentativa in range(1, self.tentativas + 1):
            self.logger.info('<<< Tentativa [%d] >>>' % (tentativa))
            try:
                self.logger.info('-->> Conectando com a SEFAZ...')
                result = requests.post(
                    url='https://' + self._servidor + os.path.join('/', self._url),
                    data=self._soap_envio.xml,
                    cert=(nome_arq_certificado.name, nome_arq_chave.name),
                    verify=nome_arq_cadeia_cert.name,
                    headers=self._soap_envio.header)
                if not result.ok:
                    razao = str(result.reason)
                    self.logger.error(
                        '<<< Tentativa [%d] - erro conexao (connect): %s-%s >>>' % (tentativa,
                                                                                    str(result.status_code),
                                                                                    razao))
                    resposta.alertas.append((tentativa, '%s-%s' % (result.status_code, result.reason)))
                    if result.status_code == 404:
                        self._soap_retorno.resposta.status = result.status_code
                        self._soap_retorno.resposta.reason = result.reason
                        break
                    time.sleep(1)
                    continue

                self._soap_retorno.resposta.xml = result.content

                # Retorno da SEFAZ
                try:
                    if self._soap_retorno.resposta.status:
                        self.logger.info('<< Status: %s-%s' % (self._soap_retorno.resposta.status,
                                                               self._soap_retorno.resposta.reason))
                except:
                    self._soap_retorno.resposta.status = ''
                if servico in (2, 3, 4, 5):  # 2=canc p/subst, 3=Inutilizar, 4=Consulta nfe, 5=Consultar servico
                    self.logger.info('<< XML resposta: %s' % self._soap_retorno.resposta.xml)
                else:
                    self.logger.info('<< XML resposta: %s' % self._soap_retorno.resposta.original)
                    self.logger.info('<< Mensagem servidor: %s' % self._soap_retorno.resposta.msg)
                break
            except Exception as err:
                if isinstance(err, str):
                    self._soap_retorno.resposta.status = 'CCC'
                    self._soap_retorno.resposta.reason = err
                    _erro = err
                    break
                elif isinstance(err.message, str):
                    self._soap_retorno.resposta.status = 'CCC'
                    self._soap_retorno.resposta.reason = err.message
                    _erro = err
                    break
                elif 'certificate verify failed' in err.message.message:
                    self._soap_retorno.resposta.status = 'CCC'
                    self._soap_retorno.resposta.reason = err.message.message
                    _erro = err.message.message
                    break
                pass
        if tentativa == self.tentativas:
            resposta.alertas.append(_erro)
        nome_arq_chave.close()
        nome_arq_certificado.close()

    def processar_lote(self, lote):
        self.logger.info('*** montagem do lote de NFC-e ***')

        # Assina e valida todas as NFC-e antes da transmissao
        lote = lote if isinstance(lote, list) else [lote]  # Sempre uma lista!
        for nfce in lote:
            nfce.chave or nfce.monta_chave()
            self.certificado.assina_xmlnfe(nfce)
            nfce.validar()
            self.logger.info('%s=> %s' % (lote.index(nfce) + 1, nfce.infNFe.Id.valor))

        envio = EnviNFe()
        resposta = RetEnviNFe()
        processo = ProcessoNFe(WS_AUTORIZACAO, envio, resposta)
        envio.NFe = lote
        envio.idLote.valor = str(self.ambiente) + datetime.now().strftime('%Y%m%d%H%M%S')
        envio.validar()
        self.logger.info('*** envio do lote de NFC-e ***')
        self._conectar_servico(WS_AUTORIZACAO, envio, resposta)

        # a partir da versao '4.47.5', quebra o envio em duas partes para nao perder o lote
        if self.__dict__.get('nova_versao'):
            return processo

        if processo.resposta.cStat.valor == '103':
            processo = self.consultar_recibo(processo.resposta.infRec.nRec.valor)

        return processo

    def consultar_nota(self, chave):
        arquivos = list()
        processo, _ = self.consultar_servico()
        if not processo.resposta.cStat.valor == '107':  # Verifica se o servico estah ativo
            return processo, _

        envio = ConsSitNFe()
        resposta = RetConsSitNFe()
        processo = ProcessoNFe(WS_NFE_CONSULTA_PROTOCOLO, envio, resposta)
        self.caminho = self.monta_caminho_nfe(chave)
        envio.tpAmb.valor = self.ambiente
        envio.chNFe.valor = chave
        envio.validar()
        novo_arquivo_nome = str(chave) + '-ped-sit.xml'
        novo_arquivo = envio.xml
        arquivos.append((novo_arquivo_nome, novo_arquivo))
        self.logger.info('*** consultar nota fiscal ***')
        self._conectar_servico(WS_NFE_CONSULTA_PROTOCOLO, envio, resposta)
        novo_arquivo_nome = str(chave) + '-sit.xml'
        novo_arquivo = resposta.xml
        arquivos.append((novo_arquivo_nome, novo_arquivo))

        return processo, arquivos
        
    def consultar_recibo(self, recibo):
        envio = ConsReciNFe()
        resposta = RetConsReciNFe()
        processo = ProcessoNFe(WS_RET_AUTORIZACAO, envio, resposta)
        envio.tpAmb.valor = self.ambiente
        envio.nRec.valor = recibo
        envio.validar()
        self.logger.info('*** consulta de lote NFC-e ***')
        self.logger.info('Lote %s' % recibo)
        for cont in range(1, 16):  # Tentativas para servico em processamento
            time.sleep(1)  # No Manual SEFAZ sao 15s
            self._conectar_servico(WS_RET_AUTORIZACAO, envio, resposta)
            if processo.resposta.cStat.valor != '105':
                break
            self.logger.info('Numero de tentativas de consulta do lote (105): %d' % cont)

        return processo

    def consultar_nfce(self, chave):
        envio = ConsSitNFe()
        resposta = RetConsSitNFe()
        processo = ProcessoNFe(WS_CONSULTA_PROTOCOLO, envio, resposta)
        envio.tpAmb.valor = self.ambiente
        envio.chNFe.valor = chave
        envio.validar()
        self.logger.info('*** consulta NFC-e ***')
        self._conectar_servico(WS_CONSULTA_PROTOCOLO, envio, resposta)

        return processo

    def cancelar_nfce(self, chave, protocolo, justificativa):
        envio = EnvEvento()
        resposta = RetEnvEvento()
        processo = ProcessoNFe(WS_RECEPCAO_EVENTO, envio, resposta)
        envio.infEvento.tpAmb.valor = self.ambiente
        envio.infEvento.chNFe.valor = chave
        envio.infEvento.nProt.valor = protocolo
        envio.infEvento.xJust.valor = justificativa
        envio.infEvento.cOrgao.valor = UF_CODIGO[self.estado]
        dt = datetime.utcnow()
        dt = dt.replace(tzinfo=pytz.utc)
        data_hora_tz = datetime.astimezone(dt, pytz.timezone("Brazil/East"))
        data_evento = datetime.strftime(data_hora_tz, '%Y-%m-%dT%H:%M:%S') + str(data_hora_tz)[-6:]
        envio.infEvento.dhEvento.valor = data_evento
        envio.infEvento.CNPJ.valor = chave[6:20]
        self.certificado.assina_xmlnfe(envio)
        envio.validar()
        self.logger.info('*** cancelamento de NFC-e ***')
        self._conectar_servico(WS_RECEPCAO_EVENTO, envio, resposta)
        if resposta.cStat.valor == '128':
            processo_cancelamento_nfe = ProcEventoNFe()
            processo_cancelamento_nfe.envEvento = envio
            processo_cancelamento_nfe.retEnvEvento = resposta
            processo_cancelamento_nfe.validar()
            processo.processo_cancelamento_nfe = processo_cancelamento_nfe

        return processo

    def cancelar_nfce_substituicao(self, chave, protocolo, justificativa, chave_ref):
        envio = EnvEventoCancSubst()
        resposta = RetEnvEventoCancSubst()
        processo = ProcessoNFe(WS_RECEPCAO_EVENTO, envio, resposta)
        envio.infEvento.tpAmb.valor = self.ambiente
        envio.infEvento.chNFe.valor = chave
        envio.infEvento.nProt.valor = protocolo
        envio.infEvento.xJust.valor = justificativa
        envio.infEvento.cOrgao.valor = UF_CODIGO[self.estado]
        envio.infEvento.cOrgaoAutor.valor = UF_CODIGO[self.estado]
        envio.infEvento.chNFeRef.valor = chave_ref
        dt = datetime.utcnow()
        dt = dt.replace(tzinfo=pytz.utc)
        data_hora_tz = datetime.astimezone(dt, pytz.timezone("Brazil/East"))
        data_evento = datetime.strftime(data_hora_tz, '%Y-%m-%dT%H:%M:%S') + str(data_hora_tz)[-6:]
        envio.infEvento.dhEvento.valor = data_evento
        envio.infEvento.CNPJ.valor = chave[6:20]
        self.certificado.assina_xmlnfe(envio)
        envio.validar()
        self.logger.info('*** cancelamento de NFC-e por substituicao ***')
        self._conectar_servico(WS_RECEPCAO_EVENTO, envio, resposta)
        if resposta.cStat.valor == '128':
            resposta.retEvento.infEvento.cOrgaoAutor.valor = UF_CODIGO[self.estado]
            processo_cancelamento_nfe = ProcEventoCancSubst()
            processo_cancelamento_nfe.envEvento = envio
            processo_cancelamento_nfe.retEnvEvento = resposta
            processo_cancelamento_nfe.validar()
            processo.processo_cancelamento_nfe = processo_cancelamento_nfe

        return processo

    def inutilizar_nfce(self, chave, justificativa):
        ano = chave[2:4]
        cnpj = chave[6:20]
        serie = chave[22:25]
        inicio = fim = chave[25:34]
        return self.inutilizar_faixa_numeracao(ano, cnpj, serie, inicio, fim, justificativa)

    def inutilizar_faixa_numeracao(self, ano, cnpj, serie, inicio=1, fim=1, justificativa=None):
        envio = InutNFe()
        resposta = RetInutNFe()
        processo = ProcessoNFe(WS_INUTILIZACAO, envio, resposta)
        envio.infInut.tpAmb.valor = self.ambiente
        envio.infInut.cUF.valor = UF_CODIGO[self.estado]
        envio.infInut.ano.valor = ano or datetime.now().strftime('%y')
        envio.infInut.CNPJ.valor = cnpj
        envio.infInut.serie.valor = int(serie)
        envio.infInut.nNFIni.valor = int(inicio)
        envio.infInut.nNFFin.valor = int(fim)
        envio.infInut.xJust.valor = justificativa
        envio.infInut.mod.valor = 65  # NFC-e
        envio.gera_nova_chave()
        self.certificado.prepara_certificado_arquivo_pfx()
        self.certificado.assina_xmlnfe(envio)
        envio.validar()
        self.logger.info('*** inutilizar faixa de numeracao NFC-e ***')
        self.logger.info('>> Faixa: %03d a %03d - Serie %s' % (int(inicio), int(fim), int(serie)))
        self.logger.info('>> ID%s (chave)' % envio.chave)
        self._conectar_servico(WS_INUTILIZACAO, envio, resposta)
        processo_inutilizacao_nfe = ProcInutNFe()

        # Se for autorizada, monta o processo de inutilização
        if resposta.infInut.cStat.valor == '102':
            processo_inutilizacao_nfe.inutNFe = envio
            processo_inutilizacao_nfe.retInutNFe = resposta
            processo_inutilizacao_nfe.validar()
            processo.processo_inutilizacao_nfe = processo_inutilizacao_nfe

        return processo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
