# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.nfe.manual_600.nfe_400 import XMLNFe, TagDecimal, TagCaracter, TagInteiro
from pynfe.nfe.manual_600.nfe_400 import NAMESPACE_NFE
from pynfe.xml_sped.assinatura import Signature


class RetInfEvento(XMLNFe):
    """Grupo de informacoes do registro do Evento"""
    def __init__(self):
        super(RetInfEvento, self).__init__()
        self.Id = TagCaracter(
            nome='infEvento', codigo='HR12', propriedade='Id', tamanho=[17, 17],
            raiz='//retEvento')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='HR13', tamanho=[1, 1], raiz='//infEvento')
        self.verAplic = TagCaracter(
            nome='verAplic', codigo='HR14', tamanho=[1, 20], raiz='//infEvento')
        self.cOrgao = TagCaracter(
            nome='cOrgao', codigo='H1R5', tamanho=[2, 2], raiz='//infEvento')
        self.cStat = TagInteiro(nome='cStat', codigo='HR16', tamanho=[3, 3], raiz='//infEvento')
        self.xMotivo = TagCaracter(
            nome='xMotivo', codigo='HR17', tamanho=[1, 255], raiz='//infEvento')
        self.chNFe = TagCaracter(
            nome='chNFe', codigo='HR18', tamanho=[44, 44], raiz='//infEvento')
        self.tpEvento = TagCaracter(
            nome='tpEvento', codigo='HR19', tamanho=[6, 6], raiz='//infEvento',
            valor='110111')
        self.xEvento = TagCaracter(
            nome='xEvento', codigo='HR20', tamanho=[5, 60], raiz='//infEvento',
            valor='CANCELAMENTO')
        self.dhRegEvento = TagCaracter(
            nome='dhRegEvento', codigo='HR25', tamanho=[0, 30], raiz='//infEvento')

    @property
    def xml(self):
        xml = self.Id.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.chNFe.xml
        xml += self.tpEvento.xml
        xml += self.xEvento.xml
        xml += self.dhRegEvento.xml
        xml += '</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cOrgao.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.chNFe.xml = arquivo
            self.tpEvento.xml = arquivo
            self.xEvento.xml = arquivo
            self.dhRegEvento.xml = arquivo


class RetEvento(XMLNFe):
    """Grupo do resultado do processamento do Evento (cancelamento)"""
    def __init__(self):
        super(RetEvento, self).__init__()
        self.versao = TagDecimal(
            nome='retEvento', codigo='HR09', propriedade='versao', valor='1.00',
            namespace=NAMESPACE_NFE, raiz='/')
        self.infEvento = RetInfEvento()

    @property
    def xml(self):
        xml = self.versao.xml
        xml += self.infEvento.xml
        xml += '</retEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo


class EventoInfEvento(XMLNFe):
    """Grupo de informacoes do evento de cancelamento"""
    def __init__(self):
        super(EventoInfEvento, self).__init__()
        self.Id = TagCaracter(
            nome='infEvento', codigo='', propriedade='Id', tamanho=[54, 54], raiz='/')

    @property
    def xml(self):
        xml = self.Id.xml
        xml += '</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo


class Evento(XMLNFe):
    """Grupo de evento"""
    def __init__(self):
        super(Evento, self).__init__()
        self.versao = TagDecimal(
            nome='evento', codigo='', propriedade='versao', valor='1.00',
            namespace=NAMESPACE_NFE, raiz='/')
        self.infEvento = EventoInfEvento()
        self.Signature = Signature()

    @property
    def xml(self):
        xml = self.versao.xml
        xml += self.infEvento.xml
        xml += self.Signature.xml
        xml += '</evento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo
            self.Signature.xml = arquivo


class ProcEventoNFe(XMLNFe):
    def __init__(self):
        super(ProcEventoNFe, self).__init__()
        self.versao = TagDecimal(
            nome='procEventoNFe', codigo='', propriedade='versao', valor='1.00',
            namespace=NAMESPACE_NFE, raiz='/')
        self.evento = Evento()
        self.retEvento = RetEvento()

    @property
    def xml(self):
        xml = self.versao.xml
        xml += self.evento.xml
        xml += self.retEvento.xml
        xml += '</procEventoNFe'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.evento.xml = arquivo
            self.retEvento.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
