# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import *
from . import ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)



class DownloadNFe(XMLNFe):
    def __init__(self):
        super(DownloadNFe, self).__init__()
        self.versao = TagDecimal(nome='downloadNFe', codigo='JP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.00', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='JP03', tamanho=[ 1, 1, 1], raiz='//downloadNFe')
        self.xServ =  TagCaracter(nome='xServ', codigo='JP04', tamanho=[12, 12, 12], raiz='//downloadNFe',valor='DOWNLOAD NFE')
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='JP05', tamanho=[ 0, 14], raiz='//downloadNFe')
        self.chNFe = TagCaracter(nome='chNFe', codigo='JP06', tamanho=[44, 44, 44], raiz='//downloadNFe')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'downloadNFe_v1.00.xsd'


    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += self.CNPJ.xml
        xml += self.chNFe.xml
        xml += '</downloadNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.xServ.xml = arquivo
            self.CNPJ.xml = arquivo
            self.chNFe.xml = arquivo




class ProcNFeGrupoZip(XMLNFe):
    def __init__(self):
        super(ProcNFeGrupoZip, self).__init__()
        self.procNFeZip = TagCaracter(nome='ProcNFeGrupoZip', codigo='JR12', tamanho=[15, 1000], raiz='//retNFe/ProcNFeGrupoZip', propriedade='procNFeZip', obrigatorio= False)
        self.NFeZip = TagCaracter(nome='NFeZip', codigo='JR14', tamanho=[44, 44, 44], raiz='//retNFe/ProcNFeGrupoZip', obrigatorio= False)
        self.protNFeZip = TagCaracter(nome='protNFeZip', codigo='JR15', tamanho=[ 0, 30],raiz='//retNFe/ProcNFeGrupoZip', obrigatorio= False)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += self.procNFeZip.xml
        xml += self.NFeZip.xml
        xml += self.protNFeZip.xml
        if  self.protNFeZip.xml:
            xml += '</ProcNFeGrupoZip>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.procNFeZip.xml = arquivo
            self.NFeZip.xml = arquivo
            self.protNFeZip.xml = arquivo



#<retDownloadNFe versao="1.00" xmlns="http://www.portalfiscal.inf.br/nfe"><tpAmb>1</tpAmb><verAplic>RS20111129150959</verAplic><cStat>139</cStat><xMotivo>Pedido de download Processado</xMotivo><dhResp>2012-12-17T17:25:53</dhResp>
#<retNFe>
#<chNFe>43121203211158000142550010000125531144040128</chNFe><cStat>140</cStat><xMotivo>Download disponibilizado</xMotivo>
#<procNFeGrupoZip>
#<NFeZip>H4sIAAAAAAAEAO29B2AcS...</NFeZip>
#<protNFeZip>H4sIAAAAAAAEAO29B2AcS...</protNFeZip>
#</procNFeGrupoZip>
#</retNFe></retDownloadNFe>





class RetNFe(XMLNFe):
    def __init__(self):
        super(RetNFe, self).__init__()
        self.chNFe = TagCaracter(nome='chNFe', codigo='JR08', tamanho=[44, 44, 44], raiz='//retDownloadNFe/retNFe')
        self.cStat = TagCaracter(nome='cStat', codigo='JR10', tamanho=[3, 3, 3], raiz='//retDownloadNFe/retNFe')
        self.xMotivo = TagCaracter(nome='xMotivo', codigo='JR11', tamanho=[1, 255], raiz='//retDownloadNFe/retNFe')
        self.procNFeGrupoZip = ProcNFeGrupoZip()

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '<retNFe>'
        xml += self.chNFe.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        if self.procNFeGrupoZip.xml:
            xml += self.procNFeGrupoZip.xml
        xml += '</retNFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.chNFe.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            if self.procNFeGrupoZip.xml:
                self.procNFeGrupoZip.xml = arquivo


class RetDownloadNFe(XMLNFe):
    def __init__(self):
        super(RetDownloadNFe, self).__init__()
        self.versao = TagDecimal(nome='retDownloadNFe',propriedade='versao', codigo='JR01', namespace=NAMESPACE_NFE, valor='1.00', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='JR03', tamanho=[ 1, 1, 1], raiz='//retDownloadNFe', valor=2)
        self.verAplic = TagCaracter(nome='verAplic', codigo='JR04', tamanho=[1, 20], raiz='//retDownloadNFe')
        self.cStat = TagCaracter(nome='cStat', codigo='JR05', tamanho=[3, 3, 3], raiz='//retDownloadNFe')
        self.xMotivo = TagCaracter(nome='xMotivo', codigo='JR06', tamanho=[1, 255], raiz='//retDownloadNFe')
        self.dhResp = TagCaracter(nome='dhResp', codigo='JR07', tamanho=[ 0, 30],raiz='//retDownloadNFe')
        self.retNFe = RetNFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retDownloadNFe_v1.00.xsd'

    @property
    def xml(self):


        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.dhResp.xml
        xml += self.retNFe.xml
        xml += '</retDownloadNFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.dhResp.xml = arquivo
            self.retNFe.xml = arquivo


