#!/usr/bin/python
# -*- encoding: utf-8 -*-
__author__ = 'thiago.rodrigues'

from pynfe.xml_sped.base import *
from . import ESQUEMA_ATUAL
import os
import gzip
from . import consnfedest
import xml.etree.ElementTree as Et
DIRNAME = os.path.dirname(__file__)


class NFeDistribuicaoDFe(XMLNFe):
    def __init__(self):
        super(NFeDistribuicaoDFe, self).__init__()
        self.versao = TagDecimal(nome='distDFeInt', codigo='A01', propriedade='versao',
                                 namespace=NAMESPACE_NFE, valor='1.01', raiz='/')
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='A03', tamanho=[1, 1, 1],
                                raiz='//distDFeInt')
        self.cUFAutor = TagCaracter(nome='cUFAutor', codigo='A04', tamanho=[0, 2],
                                    raiz='//distDFeInt')
        self.CNPJ = TagCaracter(nome='CNPJ', codigo='A05', tamanho=[0, 14],
                                raiz='//distDFeInt')
        self.distNSU = []
        self.consNSU = []
        self.consChNFe = []
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'distDFeInt_v1.01.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.cUFAutor.xml
        xml += self.CNPJ.xml
        for dist in self.distNSU:
            xml += dist.xml
        for cons in self.consNSU:
            xml += cons.xml
        for chnf in self.consChNFe:
            xml += chnf.xml
        xml += '</distDFeInt>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.cUFAutor.xml = arquivo
            self.CNPJ.xml = arquivo
            self.distNSU = self.le_grupo('//distDFeInt/distNSU', DistNSU)
            self.consNSU = self.le_grupo('//distDFeInt/consNSU', ConsNSU)
            self.consChNFe = self.le_grupo('//distDFeInt/consNFe', ConsChNFe)


class DistNSU(XMLNFe):
    def __init__(self):
        super(DistNSU, self).__init__()
        self.ultNSU = TagCaracter(nome='ultNSU', codigo='A08', tamanho=[0, 15], raiz='//distNSU')

    @property
    def xml(self):
        if not self.ultNSU.valor:
            return ''
        xml = XMLNFe.xml.fget(self)
        xml += '<distNSU>'
        xml += self.ultNSU.xml
        xml += '</distNSU>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.ultNSU.xml = arquivo



class ConsNSU(XMLNFe):
    def __init__(self):
        super(ConsNSU, self).__init__()
        self.NSU = TagCaracter(nome='NSU', codigo='A10', tamanho=[1, 15], raiz='//consNSU')

    @property
    def xml(self):
        if not self.NSU.valor:
            return ''
        xml = XMLNFe.xml.fget(self)
        xml += '<consNSU>'
        xml += self.NSU.xml
        xml += '</consNSU>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml = arquivo



class ConsChNFe(XMLNFe):
    def __init__(self):
        super(ConsChNFe, self).__init__()
        self.chNFe = TagCaracter(nome='chNFe', codigo='A12', tamanho=[44, 44],
                                 raiz='//consChNFe')

    @property
    def xml(self):
        if not self.chNFe.valor:
            return ''
        xml = XMLNFe.xml.fget(self)
        xml += '<consChNFe>'
        xml += self.chNFe.xml
        xml += '</consChNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.chNFe.xml = arquivo


class RetDistDFeInt(XMLNFe):
    def __init__(self):
        super(RetDistDFeInt, self).__init__()
        self.raiz = '//retDistDFeInt'
        self.versao = TagDecimal(nome='retDistDFeInt', propriedade='versao', codigo='B01',
                                 namespace=NAMESPACE_NFE, valor='1.00', raiz=self.raiz)
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='B03', tamanho=[1, 1, 1],
                                raiz=self.raiz)
        self.verAplic = TagCaracter(nome='verAplic', codigo='B04', tamanho=[1, 20],
                                    raiz=self.raiz)
        self.cStat = TagCaracter(nome='cStat', codigo='B05', tamanho=[3, 3, 3], raiz=self.raiz)
        self.xMotivo = TagCaracter(nome='xMotivo', codigo='B06', tamanho=[1, 255], raiz=self.raiz)
        self.dhResp = TagCaracter(nome='dhResp', codigo='B07', tamanho=[0, 30], raiz=self.raiz)
        self.ultNSU = TagCaracter(nome='ultNSU', codigo='B08', tamanho=[1, 15], raiz=self.raiz)
        self.maxNSU = TagCaracter(nome='maxNSU', codigo='B09', tamanho=[1, 15], raiz=self.raiz)
        self.loteDistDFeInt = []
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retDistDFeInt_v1.00.xsd'

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.dhResp.xml
        xml += self.ultNSU.xml
        xml += self.maxNSU.xml
        xml += '<loteDistDFeInt>'
        for lote in self.loteDistDFeInt:
            xml += lote.xml
        xml += '</loteDistDFeInt>'
        xml += '</%s>' % self.raiz.replace('//', '')
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.dhResp.xml = arquivo
            self.ultNSU.xml = arquivo
            self.maxNSU.xml = arquivo
            self.loteDistDFeInt = self.le_grupo('//loteDistDFeInt/docZip', LoteDistDFeInt)



class LoteDistDFeInt(XMLNFe):
    def __init__(self):
        super(LoteDistDFeInt, self).__init__()
        self.raiz = '/'
        self.docZip = TagCaracter(nome='docZip', codigo='B11', tamaho=[1, 50], raiz=self.raiz)
        self.NSU = TagCaracter(nome='docZip', codigo='B12', propriedade='NSU', tamanho=[1, 15],
                               raiz=self.raiz, namespace=NAMESPACE_NFE)
        self.schema = TagCaracter(nome='docZip', codigo='B13', propriedade='schema',
                                  raiz=self.raiz, namespace=NAMESPACE_NFE)
        self.resNFe = []

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        xml += '<docZip NSU="' + str(self.NSU.valor) + '" schema="' + \
               str(self.schema.valor) + '">'
        xml += str(self.docZip.valor)
        xml += '</docZip>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.docZip.xml = arquivo
            self.NSU.xml = arquivo
            self.schema.xml = arquivo



class ResNFe(consnfedest.ResNFe):
    def __init__(self):
        super(ResNFe, self).__init__()
        self.dhEmi = TagCaracter(nome='dhEmi', codigo='C08', raiz='//ret/resNFe',
                                 obrigatorio=False)
        self.nProt = TagCaracter(nome='nProt', codigo='C12', tamanho=[1, 15],
                                 raiz='//ret/resNFe')
        self.dhRecbto = TagCaracter(nome='dhRecbto', codigo='IR22', raiz='//ret/resNFe',
                                    obrigatorio=False)

    def get_res_nfe(self):
        pass

    def set_res_nfe(self, arquivo):
        self.chNFe.valor = self.get_valor(arquivo, 'chNFe')
        self.CNPJ.valor = self.get_valor(arquivo, 'CNPJ')
        self.CPF.valor = self.get_valor(arquivo, 'CPF')
        self.xNome.valor = self.get_valor(arquivo, 'xNome')
        self.IE.valor = self.get_valor(arquivo, 'IE')
        self.dhEmi.valor = self.get_valor(arquivo, 'dhEmi')
        self.tpNF.valor = self.get_valor(arquivo, 'tpNF')
        self.vNF.valor = self.get_valor(arquivo, 'vNF')
        self.digVal.valor = self.get_valor(arquivo, 'digVal')
        self.dhRecbto.valor = self.get_valor(arquivo, 'dhRecbto')
        self.nProt.valor = self.get_valor(arquivo, 'nProt')
        self.cSitNFe.valor = self.get_valor(arquivo, 'cSitNFe')

    def decodifica_nfe(self, diretorio, binario):
        import os
        nfe = binario.decode('base64')
        tmp_file = os.tempnam(diretorio, binario)
        arquivo = open(tmp_file, 'wb')
        arquivo.write(nfe)
        arquivo.close()
        nfe = gzip.open(tmp_file, 'rb')
        nfe_xml = nfe.read()
        nfe.close()
        nfe_xml = Et.fromstring(nfe_xml)
        self.set_res_nfe(nfe_xml)
        os.remove(tmp_file)

    @staticmethod
    def get_valor(arquivo, tag):
        namespace = '{'+NAMESPACE_NFE+'}'+tag
        try:
            return arquivo.find(namespace).text
        except AttributeError:
            return ''
