# -*- coding: utf-8 -*-

import os

from pynfe.xml_sped.base import TagDecimal, TagCaracter, TagInteiro, TagDataHoraUTC
from pynfe.xml_sped.base import XMLNFe, NAMESPACE_NFE
from ..manual_401 import cancnfe_200
from . import ESQUEMA_ATUAL


DIRNAME = os.path.dirname(__file__)


class InfCancEnviado(cancnfe_200.InfCancEnviado):
    def __init__(self):
        super(InfCancEnviado, self).__init__()


class CancNFe(cancnfe_200.CancNFe):
    def __init__(self):
        super(CancNFe, self).__init__()
        self.versao = TagDecimal(nome='retEvento', codigo='CP01', propriedade='versao',
                                 valor='1.00', raiz='/')
        self.infCanc = InfCancEnviado()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'cancNFe_v3.10.xsd'


class InfCancRecebido(cancnfe_200.InfCancRecebido):
    def __init__(self):
        super(InfCancRecebido, self).__init__()
        self.Id = TagCaracter(nome='infEvento', codigo='CR03', tamanho=[17, 17],
                              raiz='//retEvento', propriedade='Id', obrigatorio=False)
        self.tpAmb = TagInteiro(nome='tpAmb', codigo='CR05', tamanho=[1, 1],
                                raiz='//retEvento/infEvento', valor=2)
        self.verAplic = TagCaracter(nome='verAplic', codigo='CR06', tamanho=[1, 20],
                                    raiz='//retEvento/infEvento')
        self.cOrgao = TagInteiro(nome='cOrgao', codigo='CR12', tamanho=[2, 2],
                                 raiz='//retEvento/infEvento', obrigatorio=False)
        self.cStat = TagCaracter(nome='cStat', codigo='CR07', tamanho=[3, 3],
                                 raiz='//retEvento/infEvento')
        self.xMotivo = TagCaracter(nome='xMotivo', codigo='CR08', tamanho=[1, 255],
                                   raiz='//retEvento/infEvento')
        self.chNFe = TagCaracter(nome='chNFe', codigo='CR09', tamanho=[44, 44],
                                 raiz='//retEvento/infEvento', obrigatorio=False)
        self.tpEvento = TagInteiro(nome='tpEvento', codigo='CR13', tamanho=[6, 6],
                                   raiz='//retEvento/infEvento', obrigatorio=False)
        self.xEvento = TagCaracter(nome='xEvento', codigo='CR14', tamanho=[5, 60],
                                   raiz='//retEvento/infEvento', obrigatorio=False)
        self.CNPJDest = TagCaracter(nome='CNPJDest', codigo='CR15', tamanho=[11, 14],
                                    raiz='//retEvento/infEvento', obrigatorio=False)
        self.dhRegEvento = TagDataHoraUTC(nome='dhRegEvento', codigo='CR10',
                                          raiz='//retEvento/infEvento', obrigatorio=False)

    @property
    def xml(self):
        xml = XMLNFe.xml.fget(self)
        if self.Id.xml:
            xml += self.Id.xml
        else:
            xml += '<infEvento>'
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.chNFe.xml
        xml += self.tpEvento.xml
        xml += self.xEvento.xml
        xml += self.CNPJDest.xml
        xml += self.dhRegEvento.xml
        xml += '</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cOrgao.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.chNFe.xml = arquivo
            self.tpEvento.xml = arquivo
            self.xEvento.xml = arquivo
            self.CNPJDest.xml = arquivo
            self.dhRegEvento.xml = arquivo


class RetCancNFe(cancnfe_200.RetCancNFe):
    def __init__(self):
        super(RetCancNFe, self).__init__()
        self.versao = TagDecimal(nome='retCancNFe', codigo='CR01', propriedade='versao',
                                 valor='3.10', raiz='/')
        self.infCanc = InfCancRecebido()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retCancNFe_v3.10.xsd'

    def protocolo_cancelamento(self):
        if not self.infCanc.tpEvento.valor:
            return ''

        result = '%s - %s' % (self.infCanc.Id.valor[2:], self.infCanc.dhRegEvento.formato_danfe())

        return result


class ProcCancNFe(cancnfe_200.ProcCancNFe):
    def __init__(self):
        super(ProcCancNFe, self).__init__()
        #
        # Atenção --- a tag procCancNFe tem que começar com letra minúscula, para
        # poder validar no XSD.
        #
        self.versao = TagDecimal(
            nome='procCancNFe', propriedade='versao', namespace=NAMESPACE_NFE, valor='3.10',
            raiz='/')
        self.cancNFe = CancNFe()
        self.retCancNFe = RetCancNFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procCancNFe_v3.10.xsd'
